#!/usr/bin/env python2
"""
An example of a more complex ROS service caller
"""
import rospy
from incredible_package.srv import MatOp, MatOpResponse, MatOpRequest
# pylab package provides API similar to that of Matlab (mostly for plotting)
import pylab  # you can use matplotlib.pyplot instead
import numpy as np

SERVICE_NAME = 'mat_op'


def plot(data, subplot):
    """This is a simple plotting function

    Arguments:
        data {type: numpy array} -- points to be plotted
            2-by-N array
        subplot {string} -- specify the subplot window
            Search for matplotlib subplot for more info
    """
    pylab.subplot(subplot)
    pylab.scatter(data[0, :], data[1, :])
    pylab.xlim(-150, 150), pylab.ylim(-150, 150)


def transform(matrix, operation, *args):
    """Function that requests the specified transformation

    Arguments:
        matrix {3-by-3 numpy array} -- the input transformation matrix
        operation {string} -- the requested transformation
        args {any}  -- the arguments for the transformation

    Returns:
        type: 3-by-3 array -- the requested transformation
    """
    """
    On the following line, a request for the service is formed.
    It is simply a message. The individual message fields are provided
    directly to the constructor. The 3-by-3 matrix is flattened
    to a 1D shape.
    """
    request = MatOpRequest(operation, matrix.flatten(), args)
    # call the service with the request message and wait for response
    response = matService.call(request)
    # return the response (the matrix is reshaped from 1D to 3-by-3 shape)
    return np.reshape(response.matrixOut, (3, 3))


if __name__ == '__main__':
    rospy.wait_for_service(SERVICE_NAME)  # wait for service
    matService = rospy.ServiceProxy(SERVICE_NAME, MatOp)  # create service proxy

    n = 7  # number of points to be generated, change to your liking
    # next, generate some random points - these are 2D points in homogeneous "form"
    points = np.vstack((np.random.randint(0, 50, size=(2, n)), np.ones((1, n))))
    # create identity matrix
    matrix = np.diag([1, 1, 1])
    # plot the points
    plot(points, '151')
    # request rotation matrix by 45 degrees
    matrix = transform(matrix, 'rotate', 45)
    # apply the transformation to the points and plot the transformed points
    plot(matrix.dot(points), '152')

    # the following is the same as above only with different transformations
    matrix = transform(matrix, 'translate', 30, -30)
    plot(matrix.dot(points), '153')

    matrix = transform(matrix, 'rotate', -45)
    plot(matrix.dot(points), '154')

    matrix = transform(matrix, 'scale', 2.5, -0.5)
    plot(matrix.dot(points), '155')

    pylab.show()
