#!/usr/bin/env python2
"""
Example of ROS TF Listener

This node will check for TF between robot and the map.
Additionally, it will also listen for 2D Nav goal
and publish a TF between robot and the goal.
"""
import rospy
import tf2_ros
import numpy as np
import tf
from geometry_msgs.msg import PoseStamped, TransformStamped, Vector3, Quaternion


class GoalTF():
    def __init__(self):
        # get the necessary parameter values
        self.botFrame = rospy.get_param('bot_frame', "bot")
        self.mapFrame = rospy.get_param('map_frame', "map")
        self.goalFrame = rospy.get_param('goal_frame', "goal")
        # create subscriber for the 2D Nav goal
        self.goalSub = rospy.Subscriber("/move_base_simple/goal", PoseStamped, self.__ongoal)
        self.goal = None

        """
        Next two lines create the necessary objects for TF retrieval.
        First, we need to create a TF buffer that will store the incoming TFs,
        so that we will be able to query for them.
        The TransformListener simply subscibes to the TF topic a stores
        the TFs into the buffer.
        You can also set the buffer length as rospy.Duration(<n_secs>).
        """
        self.tf_buffer = tf2_ros.Buffer()
        self.bot_tf_listener = tf2_ros.TransformListener(self.tf_buffer)

        # TF broadcaster for the bot -> goal TF
        self.broadcaster = tf2_ros.TransformBroadcaster()

    def __ongoal(self, msg):
        """This function listens for the 2D Nav goal
        """
        self.goal = np.r_[msg.pose.position.x, msg.pose.position.y]
        rospy.loginfo("Goal was set to [{}, {}]".format(*self.goal))

    def __sendTransform(self, xdiff, ydiff):
        """Private method for broadcasting the bot-goal TF
        """
        tf_msg = TransformStamped()
        tf_msg.header.stamp = self.time.now()
        tf_msg.header.frame_id = self.botFrame
        tf_msg.child_frame_id = self.goalFrame

        # create 3D vector and fill it with pose value
        tf_msg.transform.translation = Vector3(*np.r_[xdiff, ydiff, 0])

        # no rotation needed => default quat values
        tf_msg.transform.rotation = Quaternion(0, 0, 0, 1)

        self.broadcaster.sendTransform(tf_msg)

    def update(self):
        try:
            # try to lookup transform parent -> target frame at some time
            self.time = rospy.Time()
            trans = self.tf_buffer.lookup_transform(self.mapFrame, self.botFrame, self.time)
            """
            The buffer.lookup_transform(...) function will either return a transformation
            from the parent to the target frame at the specified time or, if the transform
            does not exist, it will raise an exception.
            """
        except Exception as e:
            rospy.logerr(e)
            return False

        if self.goal is None:
            rospy.logwarn("No goal was set.")
            return False

        # Here we just compute the correct transformation parameters and broadcast them
        botPosition = np.r_[trans.transform.translation.x, trans.transform.translation.y]
        xdiff, ydiff = self.goal - botPosition
        self.__sendTransform(xdiff, ydiff)


if __name__ == "__main__":
    rospy.init_node('goaler')
    goaler = GoalTF()
    rate = rospy.Rate(10)

    while not rospy.is_shutdown():
        goaler.update()
        rate.sleep()
